@@ -36,17 +36,26 @@ PYBIND11_MODULE(_cpp_mwpm, m) {
      .def("has_computed_all_pairs_shortest_paths", &IStabiliserGraph::HasComputedAllPairsShortestPaths)
      .def("get_num_connected_components", &IStabiliserGraph::GetNumConnectedComponents);
 
+     py::class_<WeightedEdgeData>(m, "WeightedEdgeData")
+     .def_readwrite("qubit_ids", &WeightedEdgeData::qubit_ids)
+     .def_readwrite("weight", &WeightedEdgeData::weight)
+     .def_readwrite("error_probability", &WeightedEdgeData::error_probability)
+     .def_readwrite("has_error_probability", &WeightedEdgeData::has_error_probability);
+
      py::class_<WeightedStabiliserGraph, IStabiliserGraph>(m, "WeightedStabiliserGraph")
      .def(py::init<int, std::vector<int>&>(), "num_stabilisers"_a, "boundary"_a)
      .def_readwrite("all_predecessors", &WeightedStabiliserGraph::all_predecessors)
      .def_readwrite("all_distances", &WeightedStabiliserGraph::all_distances)
      .def_readwrite("all_edges_have_error_probabilities", 
                     &WeightedStabiliserGraph::all_edges_have_error_probabilities)
+     .def_readwrite("_distances", &WeightedStabiliserGraph::_distances)
+     .def_readwrite("_predecessors", &WeightedStabiliserGraph::_predecessors)
      .def("add_edge", &WeightedStabiliserGraph::AddEdge, "node1"_a, "node2"_a, "qubit_ids"_a, 
           "weight"_a, "error_probability"_a=-1.0, "has_error_probability"_a=false)
      .def("add_noise", &WeightedStabiliserGraph::AddNoise)
      .def("get_boundary", &WeightedStabiliserGraph::GetBoundary)
      .def("set_boundary", &WeightedStabiliserGraph::SetBoundary, "boundary"_a)
+     .def("get_edges", &WeightedStabiliserGraph::GetEdges)
      .def("get_nearest_neighbours", &WeightedStabiliserGraph::GetNearestNeighbours,
           "source"_a, "num_neighbours"_a, "defect_id"_a)
      .def("get_path", &WeightedStabiliserGraph::GetPath, "source"_a, "target"_a);
@@ -55,6 +64,13 @@ PYBIND11_MODULE(_cpp_mwpm, m) {
      m.def("set_seed", &set_seed, "s"_a);
      m.def("rand_float", &rand_float, "from"_a, "to"_a);
 
-     m.def("decode_match_neighbourhood", &LemonDecodeMatchNeighbourhood);
-     m.def("decode", &LemonDecode);
+     py::class_<MatchingResult>(m, "MatchingResult")
+     .def_readwrite("correction", &MatchingResult::correction)
+     .def_readwrite("weight", &MatchingResult::weight);
+
+     m.def("decode_match_neighbourhood", &LemonDecodeMatchNeighbourhood,
+           "sg"_a, "defects"_a, "num_neighbours"_a=20,
+           "return_weight"_a=false);
+     m.def("decode", &LemonDecode, "sg"_a, "defects"_a,
+           "return_weight"_a=false);
 }
