@@ -12,6 +12,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import warnings
+
+import matplotlib.cbook
 import numpy as np
 import networkx as nx
 from scipy.sparse import csc_matrix, spmatrix, vstack
@@ -21,6 +24,7 @@ from pymatching._cpp_mwpm import (decode,
                             WeightedStabiliserGraph)
 
 
+
 def _find_boundary_nodes(G):
     """Find all boundary nodes in G
 
@@ -114,8 +118,8 @@ class Matching:
                 H = csc_matrix(H)
                 unique_elements = np.unique(H.data)
                 if len(unique_elements)>1 or unique_elements[0] != 1:
-                    raise ValueError("Nonzero elements in the parity check matrix"
-                                    f" must be 1, not {unique_elements}.")
+                    raise ValueError("Nonzero elements in the parity check matrix"\
+                                    " must be 1, not {}.".format(unique_elements))
                 H = H.astype(np.uint8)
             except:
                 raise ValueError("H must be a NetworkX graph or convertible "
@@ -139,8 +143,8 @@ class Matching:
             column_weights = np.asarray(H.sum(axis=0))[0]
             unique_column_weights = np.unique(column_weights)
             if np.setdiff1d(unique_column_weights, np.array([1,2])).size > 0:
-                raise ValueError("Each qubit must be contained in either "
-                                 f"1 or 2 check operators, not {unique_column_weights}")
+                raise ValueError("Each qubit must be contained in either "\
+                                 "1 or 2 check operators, not {}".format(unique_column_weights))
             H.eliminate_zeros()
             H.sort_indices()
             self.num_stabilisers = H.shape[0]
@@ -176,9 +180,9 @@ class Matching:
             for (u, v, attr) in H.edges(data=True):
                 u, v = int(u), int(v)
                 if u >= num_nodes or v>= num_nodes:
-                    raise ValueError("Every node id must be less "
-                        "than the number of nodes, but edge "
-                        f"({u},{v}) was present.")
+                    raise ValueError("Every node id must be less "\
+                        "than the number of nodes, but edge "\
+                        "({},{}) was present.".format(u,v))
                 qubit_id = attr.get("qubit_id", set())
                 if isinstance(qubit_id, (int, np.integer)):
                     qubit_id = {int(qubit_id)} if qubit_id != -1 else set()
@@ -186,10 +190,10 @@ class Matching:
                     try:
                         qubit_id = set(qubit_id)
                         if not all(isinstance(q, (int, np.integer)) for q in qubit_id):
-                            raise ValueError(f"qubit_id must be a set of ints, not {qubit_id}")
+                            raise ValueError("qubit_id must be a set of ints, not {}".format(qubit_id))
                     except:
-                        raise ValueError("qubit_id property must be an int or a set of int"
-                                f" (or convertible to a set), not {qubit_id}")
+                        raise ValueError("qubit_id property must be an int or a set of int"\
+                                " (or convertible to a set), not {}".format(qubit_id))
                 all_qubits = all_qubits | qubit_id
                 weight = attr.get("weight", 1) # Default weight is 1 if not provided
                 if weight < 0:
@@ -198,12 +202,12 @@ class Matching:
                 g.add_edge(u, v, qubit_id, weight, e_prob, 0<=e_prob<=1)
             self.stabiliser_graph = g
             if max(all_qubits, default=-1) != len(all_qubits) - 1:
-                raise ValueError(f"The maximum qubit id ({max(all_qubits, default=0)}) should "
-                        f"equal the number of qubits ({len(all_qubits)}) minus one.")
+                raise ValueError("The maximum qubit id ({}) should "\
+                        "equal the number of qubits ({}) minus one.".format(max(all_qubits, default=0),len(all_qubits)))
         num_components = self.stabiliser_graph.get_num_connected_components()
         if num_components != 1:
-            raise ValueError("Matching graph must have 1 connected component, "
-                             f"but instead has {num_components}")
+            raise ValueError("Matching graph must have 1 connected component, "\
+                             "but instead has {}".format(num_components))
         if precompute_shortest_paths:
             self.stabiliser_graph.compute_all_pairs_shortest_paths()
     
@@ -222,7 +226,7 @@ class Matching:
         """
         return self.stabiliser_graph.get_boundary()
     
-    def decode(self, z, num_neighbours=20):
+    def decode(self, z, num_neighbours=20, return_weight=False):
         """Decode the syndrome `z` using minimum-weight perfect matching
 
         If the parity of `z` is odd, then the first boundary node in 
@@ -245,13 +249,16 @@ class Matching:
             https://arxiv.org/abs/2010.09626 is used, and `num_neighbours` 
             corresponds to the parameter `m` in the paper. It is recommended 
             to leave `num_neighbours` set to at least 20.
-            If `num_neighbours=None`, then instead full matching is 
+            If `num_neighbours is None`, then instead full matching is 
             performed, with the all-pairs shortest paths precomputed and 
             cached the first time it is used. Since full matching is more 
             memory intensive, it is not recommended to be used for matching graphs 
             with more than around 10,000 nodes, and is only faster than 
             local matching for matching graphs with less than around 1,000 
             nodes. By default 20
+        return_weight : bool, optional
+            If `return_weight==True`, the sum of the weights of the edges in the 
+            minimum weight perfect matching is also returned. By default False
 
         Returns
         -------
@@ -261,12 +268,17 @@ class Matching:
             and an element is 1 if the corresponding qubit should be flipped, 
             and otherwise 0.
 
+        float
+            Present only if `return_weight==True`.
+            The sum of the weights of the edges in the minimum-weight perfect 
+            matching.
+
         """
         try:
             z = np.array(z, dtype=np.uint8)
         except:
             raise ValueError("Syndrome must be of type numpy.ndarray or "\
-                             f"convertible to numpy.ndarray, not {z}")
+                             "convertible to numpy.ndarray, not {}".format(z))
         if len(z.shape) == 1 and (self.num_stabilisers <= z.shape[0]
                 <= self.num_stabilisers+len(self.boundary)):
             defects = z.nonzero()[0]
@@ -274,21 +286,25 @@ class Matching:
             num_stabs = self.stabiliser_graph.get_num_nodes()
             max_num_defects = z.shape[0]*z.shape[1]
             if max_num_defects > num_stabs:
-                raise ValueError(f"Syndrome size {z.shape[0]}x{z.shape[1]} exceeds" \
-                        f" the number of stabilisers ({num_stabs})")
+                raise ValueError("Syndrome size {}x{} exceeds" \
+                        " the number of stabilisers ({})".format(z.shape[0],z.shape[1],num_stabs))
             times, checks = z.T.nonzero()
             defects = times*self.num_stabilisers + checks
         else:
-            raise ValueError(f"The shape ({z.shape}) of the syndrome vector z is not valid.")
+            raise ValueError("The shape ({}) of the syndrome vector z is not valid.".format(z.shape))
         if len(defects) % 2 != 0:
             if len(self.boundary) == 0:
                 raise ValueError("Syndrome must contain an even number of defects "
                                     "if no boundary vertex is given.")
             defects = np.setxor1d(defects, np.array(self.boundary[0:1]))
         if num_neighbours is None:
-            return decode(self.stabiliser_graph, defects)
+            res = decode(self.stabiliser_graph, defects, return_weight)
         else:
-            return decode_match_neighbourhood(self.stabiliser_graph, defects, num_neighbours)
+            res = decode_match_neighbourhood(self.stabiliser_graph, defects, num_neighbours, return_weight)
+        if return_weight:
+            return res.correction, res.weight
+        else:
+            return res.correction
     
     def add_noise(self):
         """Add noise by flipping edges in the stabiliser graph with 
@@ -313,13 +329,89 @@ class Matching:
             return None
         return self.stabiliser_graph.add_noise()
     
+    def edges(self):
+        """Edges of the matching graph
+
+        Returns a list of edges of the matching graph. Each edge is a 
+        tuple `(source, target, attr)` where `source` and `target` are ints corresponding to the 
+        indices of the source and target nodes, and `attr` is a dictionary containing the 
+        attributes of the edge.
+        The dictionary `attr` has keys `qubit_id` (a set of ints), `weight` (the weight of the edge, 
+        set to 1.0 if not specified), and `error_probability` 
+        (the error probability of the edge, set to -1 if not specified).
+
+        Returns
+        -------
+        List of (int, int, dict) tuples
+            A list of edges of the matching graph
+        """
+        edata = self.stabiliser_graph.get_edges()
+        return [(e[0], e[1], {
+            'qubit_id': e[2].qubit_ids,
+            'weight': e[2].weight,
+            'error_probability': e[2].error_probability
+            }) for e in edata]
+    
+    def to_networkx(self):
+        """Convert to NetworkX graph
+
+        Returns a NetworkX graph corresponding to the matching graph. Each edge 
+        has attributes `qubit_ids`, `weight` and `error_probability` and each node has 
+        the attribute `is_boundary`.
+
+        Returns
+        -------
+        NetworkX.Graph
+            NetworkX Graph corresponding to the matching graph
+        """
+        G = nx.Graph()
+        G.add_edges_from(self.edges())
+        boundary = self.boundary
+        for i in range(G.number_of_nodes()):
+            is_boundary = i in boundary
+            G.nodes[i]['is_boundary'] = is_boundary
+        return G
+    
+    def draw(self):
+        """Draw the matching graph using matplotlib
+
+        Draws the matching graph as a matplotlib graph. Stabiliser nodes are 
+        filled grey and boundary nodes are filled white. The line thickness of each 
+        edge is determined from its weight (with min and max thicknesses of 0.2 pts
+        and 2 pts respectively).
+        Note that you may need to call `plt.figure()` before and `plt.show()` after calling 
+        this function.
+        """
+        # Ignore matplotlib deprecation warnings from networkx.draw_networkx
+        warnings.filterwarnings("ignore",category=matplotlib.cbook.mplDeprecation)
+        warnings.filterwarnings("ignore", category=DeprecationWarning)
+        G = self.to_networkx()
+        pos=nx.spectral_layout(G, weight=None)
+        c = "#bfbfbf"
+        ncolors = ['w' if n[1]['is_boundary'] else c for n in G.nodes(data=True)]
+        nx.draw_networkx_nodes(G, pos=pos, node_color=ncolors, edgecolors=c)
+        nx.draw_networkx_labels(G, pos=pos)
+        weights=np.array([e[2]['weight'] for e in G.edges(data=True)])
+        normalised_weights = 0.2+2*weights/np.max(weights)
+        nx.draw_networkx_edges(G, pos=pos, width=normalised_weights)
+        def qid_to_str(qid):
+            if len(qid) == 0:
+                return ""
+            elif len(qid) == 1:
+                return str(qid.pop())
+            else:
+                return str(qid)
+        edge_labels = {(s, t): qid_to_str(d['qubit_id']) for (s,t,d) in G.edges(data=True)}
+        nx.draw_networkx_edge_labels(G, pos=pos, edge_labels=edge_labels)
+
     def __repr__(self):
         N = self.num_qubits
         M = self.num_stabilisers
         B = len(self.boundary)
         E = self.stabiliser_graph.get_num_edges()
-        return (f"<pymatching.Matching object with "
-                f"{N} qubit{'s' if N != 1 else ''},"
-                f" {M} stabiliser{'s' if M != 1 else ''}, "
-                f"{B} boundary node{'s' if B != 1 else ''}, and "
-                f"{E} edge{'s' if E != 1 else ''}>")
+        return "<pymatching.Matching object with "\
+                "{} qubit{}, {} stabiliser{}, "\
+                "{} boundary node{}, "\
+                "and {} edge{}>".format(N, 's' if N != 1 else '',
+                    M, 's' if M != 1 else '', B, 's' if B != 1 else '',
+                    E, 's' if E != 1 else '')
